<?php
/**
 * Helper
 * Create a chatbot with AI features for your website.
 * Exclusively on https://1.envato.market/helper
 *
 * @encoding        UTF-8
 * @version         1.0.1
 * @copyright       (C) 2018 - 2023 Merkulove ( https://merkulov.design/ ). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Cherviakov Vlad (vladchervjakov@gmail.com), Nemirovskiy Vitaliy (nemirovskiyvitaliy@gmail.com), Dmytro Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

namespace Merkulove\Helper;

use Merkulove\Helper\Unity\Plugin;
use Merkulove\Helper\Unity\Settings;
use Merkulove\Helper\Unity\TabAssignments;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Caster class contain main plugin logic.
 *
 * @since 1.0.0
 *
 **/
final class Caster {

	/**
	 * The one true Caster.
	 *
     * @since 1.0.0
     * @access private
	 * @var Caster
	 **/
	private static $instance;

    /**
     * Setup the plugin.
     *
     * @since 1.0.0
     * @access public
     *
     * @return void
     **/
    public function setup() {

        /** Define hooks that runs on both the front-end as well as the dashboard. */
        $this->both_hooks();

        /** Define public hooks. */
        $this->public_hooks();

        /** Define admin hooks. */
        $this->admin_hooks();

    }

    /**
     * Define hooks that runs on both the front-end as well as the dashboard.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function both_hooks() {

        /** Hooks which should work everywhere in backend and frontend. */
        if ( Settings::get_instance()->options['bot_position'] === 'shortcode' )
        add_shortcode( 'helper', [$this, 'helper_shortcode'] );

        /** Add ajax actions */
        AjaxActions::get_instance();


    }

    /**
     * Register all of the hooks related to the public-facing functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function public_hooks() {

        /** Public hooks and filters. */

        /** Work only on frontend area. */
        if ( is_admin() ) { return; }

        /** Add front styles and scripts */
        FrontStyles::get_instance();
        FrontScripts::get_instance();

        if ( Settings::get_instance()->options['bot_position'] !== 'shortcode' ) {
            add_action( 'wp_footer', [$this, 'the_chat_bot'] );
        }

    }
    /**
     * Get svg code of the svg file
     * @param $file
     *
     * @return mixed|string
     */
    public function get_svg( $file ) {

        $file_content = wp_remote_get( Plugin::get_url() . 'images/mdc-icons/' . $file, [
            'sslverify'  => Settings::get_instance()->options[ 'check_ssl' ] === 'on'
        ] );

        return is_array( $file_content ) && isset( $file_content[ 'body' ] ) ? $file_content[ 'body' ] : '';

    }


    /**
     * Register all of the hooks related to the admin area functionality.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    private function admin_hooks() {

        /** Admin hooks and filters here. */

        $options = Settings::get_instance()->options;

        $collect_data_enabled = !empty( $options['save_user_data'] ) &&
            $options['save_user_data'] === 'on' &&
            in_array( 'collect_data', $options['bot_features'] );

        /** Work only in admin area. */
        if ( ! is_admin() ) { return; }

        if ( $collect_data_enabled ) {

            /** Create user data record post type */
            add_action( 'init', [ UserData::get_instance(), 'create_user_data_cpt' ] );

            /** Remove admin menu added from Unity. */
            add_action( 'admin_menu', [ $this, 'remove_admin_menu' ], 1000 );
            add_action( 'admin_menu', [ $this, 'add_admin_menu' ], 1001 );

        }

        /** Create Helper receive and send message sounds dir. */
        wp_mkdir_p( trailingslashit( wp_upload_dir()['basedir'] ) . trailingslashit( 'helper' ) . 'send_message_sound' );
        wp_mkdir_p( trailingslashit( wp_upload_dir()['basedir'] ) . trailingslashit( 'helper' )  . 'receive_message_sound' );

        /** Create message sound file */
        add_action( 'wp_ajax_mdp_helper_save_sound', [$this, 'save_sound_file'] );

        /** Clear message sound file */
        add_action( 'wp_ajax_mdp_helper_remove_audio', [$this, 'clear_sound_file'] );

    }

	/**
	 * This method used in register_activation_hook
	 * Everything written here will be done during plugin activation.
	 *
	 * @since 1.0.0
	 * @access public
	 */
	public function activation_hook() {

		/** Activation hook */

	}

    /**
     * Display chatbot with shortcode.
     *
     * @since 1.0.0
     * @access public
     */
    public function helper_shortcode() {
        ob_start();
        $this->the_chat_bot();
        return ob_get_clean();
    }

    /**
     * Save file receive/send message sound from settings.
     *
     * @since 1.0.0
     * @access public
     */
    public function save_sound_file() {
        /** Check nonce for security. */
        check_ajax_referer( 'helper-unity', 'nonce' );

        $file_name = esc_attr( $_POST['mdp_sound_file_name'] );
        $type = esc_attr( $_POST['mdp_sound_type'] );

        $mime = mime_content_type( $_FILES['mdp_sound_file']['tmp_name'] );

            if ( !in_array( $mime, [ 'audio/mpeg', 'audio/wav', 'audio/ogg' ] ) || empty( $file_name ) || empty( $type ) ) {
            wp_delete_file( $_FILES['mdp_sound_file']['tmp_name'] );
            wp_send_json_error( esc_html__( 'Error occurred while saving file!', 'helper' ) );
        }

        $upload_dir     = wp_get_upload_dir();
        $upload_basedir = $upload_dir['basedir'];

        $audio_file = $upload_basedir . '/helper/' . $type . '/' . $file_name;

        $file_result = file_put_contents( $audio_file, file_get_contents( $_FILES['mdp_sound_file']['tmp_name'] ) );

        if ( !$file_result ) {
            wp_send_json_error( esc_html__( 'Error occurred while saving file!', 'helper' ) );
        }

    }

    /**
     * Remove file receive/send message sound from folder.
     *
     * @since 1.0.0
     * @access public
     */
    public function clear_sound_file() {
        /** Check nonce for security */
        check_ajax_referer( 'helper-unity', 'nonce' );

        $type = esc_attr( $_POST['mdp_sound_type'] );
        $file_name = esc_attr( $_POST['mdp_file_name'] );

        if ( empty( $type ) || empty( $file_name ) ) { wp_send_json_error( 'Error while resetting sound!' ); }

        $upload_dir     = wp_get_upload_dir();
        $upload_basedir = $upload_dir['basedir'];

        $audio_file = $upload_basedir . '/helper/' . $type . '/' . $file_name;

        wp_delete_file( $audio_file );

        wp_send_json_success( $upload_basedir . '/helper/' . $type . '/' . $file_name );

    }

    /**
     * Add admin menu for plugin settings.
     *
     * @since 1.0.0
     * @access public
     **/
    public function add_admin_menu() {

        add_submenu_page(
            'edit.php?post_type=mdp_user_data_cpt',
            esc_html__( 'Helper Settings', 'helper' ),
            esc_html__( 'Settings', 'helper' ),
            'manage_options',
            'mdp_helper_settings',
            [Settings::get_instance(), 'options_page']
        );

    }

    /**
     * Remove admin menu added from Unity.
     *
     * @since 1.0.0
     * @access private
     *
     * @return void
     **/
    public function remove_admin_menu() {

        /** Remove menu item by slug. */
        remove_menu_page( 'mdp_helper_settings' );

    }

    /**
     * Returns data from repeater fields.
     *
     * @since 1.0.0
     * @access private
     **/
    public function get_all_repeater_data( $repeater_size, $field_name ) {
        $options = Settings::get_instance()->options;
        $repeater_data = [];

        for ( $i = 0; $i < $repeater_size; $i++ ) {
            if ( empty( $options[$field_name . $i] ) ) { continue; }
            $repeater_data[] = trim( esc_attr__( $options[$field_name . $i], 'helper' ) );
        }

        return $repeater_data;
    }

    /**
     * Creates header of chat box.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_chat_header() {

        $options = Settings::get_instance()->options;
        $enabled_bot_commands = isset( $options['bot_features'] ) && in_array( 'bot_commands', $options['bot_features'] );
		$chat_header = $options[ 'chat_header_heading' ] === 'on' ? $options[ 'chat_header_text' ] : '';

        return sprintf(
            '<div class="mdp-helper-chatbot-header%s">
                       %s
                    </div>',
           $enabled_bot_commands ? ' mdp-helper-chatbot-header-space-between' : '',
				   $chat_header,

        );

    }

	/**
	 * Single toolbar button.
	 *
	 * @param $key
	 * @param $options
	 * @param string $type
	 *
	 * @return string
	 */
	private function toolbar_button( $key, $options, string $type = 'link' ): string {

		$is_enable = $options[ $key . '_enable' ] ?? 'off';
		if ( $is_enable !== 'on' ) { return ''; }

		switch( $type ) {

			case 'link':

				/** @noinspection HtmlUnknownTarget */
				return wp_sprintf(
					'<a href="%4$s" class="mdp-helper-toolbar-link mdp-helper-%1$s" target="_blank">
						<span class="mdp-helper-toolbar-icon mdp-helper-%1$s-icon">%2$s</span>
						<span class="mdp-helper-toolbar-caption mdp-helper-%1$s-caption">%3$s</span>
					</a>',
					str_replace( '_', '-', $key ),
					$this->get_svg( $options[ $key . '_icon' ] ),
					$options[ $key . '_text' ],
					esc_url( $options[ $key . '_link' ] )
				);

			case 'button':

				return wp_sprintf(
					'<button class="mdp-helper-toolbar-button mdp-helper-%1$s" title="%3$s">
						<span class="mdp-helper-toolbar-icon mdp-helper-%1$s-icon">%2$s</span>
						<span class="mdp-helper-toolbar-caption mdp-helper-%1$s-caption">%3$s</span>
					</button>',
					str_replace( '_', '-', $key ),
					$this->get_svg( $options[ $key . '_icon' ] ),
					$options[ $key . '_text' ],
				);

			default:

				return '';

		}



	}

	/**
	 * Creates toolbar of chat box.
	 * @return string
	 */
	private function create_chat_toolbar(): string {

		$options = Settings::get_instance()->options;

		$enabled_bot_commands = isset( $options['bot_features'] ) && in_array( 'bot_commands', $options['bot_features'] );

		$close_bot_button = $options['open_bot_with_button'] === 'on' && $options[ 'close_button_show' ] === 'on' ?
			sprintf(
				'<button class="mdp-helper-chatbot-close-button">%s</button>',
				!empty( $options[ 'close_button_icon' ] ) ? $this->get_svg( $options[ 'close_button_icon' ] ) : ''
			) : '';

		return wp_sprintf(
			'<div class="mdp-helper-chatbot-toolbar">
				%s
				%s
				%s
				%s
				%s
				%s
				%s
			</div>',
			$enabled_bot_commands ? $this->toolbar_button( 'bot_commands', $options, 'button' ) : '',
			$this->toolbar_button( 'mute_button', $options, 'button' ),
			$this->toolbar_button( 'mail_button', $options ),
			$this->toolbar_button( 'call_button', $options ),
			$this->toolbar_button( 'social_button', $options ),
			$this->toolbar_button( 'messenger_button', $options ),
			$close_bot_button
		);

	}

    /**
     * Creates chat container of chat box.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_chat_container(): string {

        return '<div class="mdp-helper-chatbot-messages">
					<div class="mdp-helper-messages-wrapper">
	                	<div class="mdp-helper-chatbot-messages-container"></div>
	            </div>
        	</div>';

    }

	/**
	 * Send message button markup.
	 * @param $options
	 *
	 * @return string
	 */
	private function send_button_markup( $options ): string {

		if ( $options[ 'send_button_show' ] !== 'on' ) { return ''; }

		// CSS class for overlap button.
		$overlap = $options[ 'send_button_overlap' ] === 'on' ? ' mdp-helper-bot-button-overlap' : '';

		// Send button icon.
		$button_icon = '';
		$svg_markup = $this->get_svg( $options[ 'send_button_icon' ] );
		if ( $svg_markup !== '' ) {
			$button_icon = wp_sprintf(
				'<span class="mdp-helper-send-message-button-icon">%s</span>',
				$svg_markup
			);
		}

		// Send button caption.
		$button_caption = '';
		if ( $options[ 'send_button_caption' ] !== '' ) {
			$button_caption = wp_sprintf(
				'<span class="mdp-helper-send-message-button-caption">%s</span>',
				$options[ 'send_button_caption' ]
			);
		}

		return wp_sprintf(
			'<button class="mdp-helper-send-message-button mdp-helper-bot-button%s" title="%s">%s%s</button>',
			$overlap,
			esc_attr__( $options[ 'send_button_caption' ], 'helper' ),
			$button_icon,
			$button_caption,
		);

	}

    /**
     * Creates footer of chat box.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_chat_footer() {

		$options = Settings::get_instance()->options;

		// Check if footer text is enabled.
		$footer_text = '';
		if ( $options[ 'chat_footer_text_show' ] === 'on' ) {
			$footer_text = wp_sprintf(
				'<div class="mdp-helper-chatbot-footer-text">%s</div>',
				wp_kses_post( $options[ 'chat_footer_text_message' ] )
			);
		}

		// Speech recognition button icon.
	    $recognition_button = '';
		if ( $options[ 'bot_stt' ] === 'on' ) {
			$recognition_button = wp_sprintf(
				'<button class="mdp-helper-bot-button mdp-helper-bot-button-recognize" title="%1$s" data-in-progress="false">
					%2$s
				</button>',
				esc_attr__( 'Recognize voice message', 'helper' ),
				$this->get_svg( $options[ 'bot_stt_icon' ] )
			);
		}

        return sprintf(
            '<div class="mdp-helper-chatbot-footer">
                <div class="mdp-helper-chatbot-footer-form mdp-helper-form-disabled%s">
                	%s
                    <input id="mdp-helper-input-messages-field" class="mdp-helper-input-messages-field" placeholder="%s" type="text">
                    %s
				</div>
                %s
            </div>',
	        $options[ 'bot_stt' ] === 'on' ? ' mdp-helper-form-with-recognize' : '',
			$recognition_button,
	        ! empty( $options[ 'send_button_placeholder' ] ) ? $options[ 'send_button_placeholder' ] : esc_html__( 'Type your message', 'helper' ),
			$this->send_button_markup( $options ),
            $footer_text
        );
    }

    /**
     * Creates chat box.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_chatbot_box() {
        $options = Settings::get_instance()->options;

        return sprintf(
            '<div class="mdp-helper-chatbot-box %s">%s %s %s %s</div>',
            $options['open_bot_with_button'] === 'on' ? 'mdp-helper-hide-chat' : '',
                    $this->create_chat_header(),
					$this->create_chat_toolbar(),
                    $this->create_chat_container(),
                    $this->create_chat_footer()
        );
    }

    /**
     * Creates chatbot button.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function create_chat_bot_button(): string {
        $options = Settings::get_instance()->options;

        if ( $options['open_bot_with_button'] !== 'on' ) { return ''; }

		$icon = '';
		if ( $options[ 'open_bot_button_enable_icon' ] === 'on' ) {
			$icon = ! empty( $options['open_bot_button_icon'] ) ? $this->get_svg( $options['open_bot_button_icon'] ) : '';
		}

		$caption = '';
		if ( $options[ 'open_bot_button_enable_caption' ] === 'on' ) {
			$caption = ! empty( $options['open_bot_button_caption'] ) ? esc_html__( $options['open_bot_button_caption'], 'helper' ) : '';
		}

        return sprintf(
            '<div class="mdp-helper-open-button-wrapper mdp-helper-open-button-alignment-%s">
                <button id="mdp-helper-open-button" class="mdp-helper-open-chatbot-button%s" title="%s">
                    %s%s
                </button>
            </div>',
            esc_attr( $options['open_bot_button_alignment'] ),
	        $options[ 'open_bot_button_aura' ] === 'on' ? ' mdp-helper-open-button-aura' : '',
	        $options['open_bot_button_caption'] !== '' ? esc_attr( $options['open_bot_button_caption'] ) : esc_html__( 'Open chat', 'helper'),
            $icon !== '' ? '<span class="mdp-helper-open-button-icon">' . $icon . '</span>' : '',
            $caption !== '' ? '<span class="mdp-helper-open-button-caption">' . $caption . '</span>' : ''
        );
    }

    /**
     * Creates chat bot.
     *
     * @return void
     * @since 1.0.0
     * @access private
     *
     */
    public function the_chat_bot() {

        /** Checks if plugin should work on this page. */
        if ( ! TabAssignments::get_instance()->display() ) { echo ''; }

        $options = Settings::get_instance()->options;
        $bot_position = esc_attr( $options['bot_position'] );
        $is_bottom = $bot_position === 'fixed-bottom-left' || $bot_position === 'fixed-bottom-center' || $bot_position === 'fixed-bottom-right';

        echo sprintf(
            '<div class="mdp-helper-box mdp-helper-bot-position-%s" style="display: none;">%s %s</div>%s',
            $bot_position,
                    $is_bottom ? $this->create_chatbot_box() : $this->create_chat_bot_button(),
                    $is_bottom ? $this->create_chat_bot_button() : $this->create_chatbot_box(),
                    HelperStyles::get_instance()->create_styles()
        );
    }

	/**
	 * Main Caster Instance.
	 * Insures that only one instance of Caster exists in memory at any one time.
	 *
	 * @static
     * @since 1.0.0
     * @access public
     *
	 * @return Caster
	 **/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

			self::$instance = new self;

		}

		return self::$instance;

	}

}
