<?php
namespace Merkulove\Helper;

use Merkulove\Helper\Unity\Plugin;
use Merkulove\Helper\Unity\Settings;
use Orhanerday\OpenAi\OpenAi;


/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
    header( 'Status: 403 Forbidden' );
    header( 'HTTP/1.1 403 Forbidden' );
    exit;
}

require Plugin::get_path() . '/vendor/autoload.php';

/**
* SINGLETON: AjaxActions class contains ajax logic.
*
* @since 1.0.0
*
**/
final class OpenAiBot {

    /**
     * The one true AjaxActions.
     *
     * @since 1.0.0
     * @access private
     * @var OpenAiBot
     **/
    private static $instance;



    /**
     * Creates prompt based on post content.
     *
     * @return string
     * @since 1.0.0
     * @access private
     *
     */
    private function get_post_content() {
        $options = Settings::get_instance()->options;
        if ( empty( $options['open_ai_post_id'] ) ) { return ''; }

        $posts_content = '';

        $posts_ids = $options['open_ai_post_id'];

        foreach ( $posts_ids as $posts_id ) {
            $post_content = get_post( esc_attr( str_replace( 'post_', '', $posts_id ) ) )->post_content;
            $post_content = apply_filters( 'the_content', $post_content );
            $post_content = str_replace(']]>', ']]&gt;', $post_content );
            $posts_content .= strip_tags( $post_content );
            $posts_content .= ' ';
        }

        return $posts_content;

    }

    /**
     * Returns product categories
     *
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function get_product_categories( $product_id ) {
        $product_cats = get_the_terms( $product_id, 'product_cat' );
        $categories = [];

        foreach ( $product_cats as $product_cat ) {
            $categories[] = $product_cat->name;
        }

        return implode( ', ', $categories );
    }

    /**
     * Returns prompt based on WooCommerce products
     *
     * @return string
     * @since 1.0.0
     * @access private
     */
    private function get_products_prompt() {
        if (  !in_array('woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
            return '';
        }

        $products = wc_get_products( [ 'status' => 'publish', 'limit' => -1  ] );

        $products_prompt = 'Products: ';

        if ( !empty( $products ) ) {
            foreach ( $products as $product ) {
                $product = $product->get_data();
                $product_cats = $this->get_product_categories( $product['id'] );
                $products_prompt .= sprintf(
                    'Name: %s, Description: %s, Category: %s, Price: %s%s,
                            In stock: %s, On sale: %s, Link: %s ',
                    $product['name'],
                            $product['description'],
                            $product_cats,
                            $product['price'],
                            get_woocommerce_currency_symbol(),
                            $product['stock_status']=== 'instock' ? 'yes' : 'no',
                            !empty( $product['sale_price'] ) ? 'yes' : 'no',
                            get_permalink( $product['id'] )
                );
            }
        }

        return $products_prompt;
    }

    /**
     * Return respond from Open AI
     *
     * @return string
     * @throws \Exception
     * @since 1.0.0
     * @access private
     */
    public function get_open_ai_bot_result( $question ) {
        $options = Settings::get_instance()->options;

        /** If key not provided return */
        if ( empty( $options['open_ai_api_key'] ) ) { return ''; }

        $open_ai = new OpenAi( $options['open_ai_api_key'] );

        $prompt = '';
        $objective = "\n\nObjective:" . esc_html( $options['open_ai_objective'] );
        $notice = "\n\nNotice:" . esc_html( $options['open_ai_notice'] );

        if ( in_array( 'custom', $options['open_ai_prompt_type'] ) ) {

            $description = esc_html( $options['open_ai_prompt'] );

            $prompt .= $description;

        }
        if ( in_array( 'woocommerce_products', $options['open_ai_prompt_type'] ) ) {
            $prompt .= $this->get_products_prompt();
        }
        if ( in_array( 'post_content', $options['open_ai_prompt_type'] )  ) {
            $prompt .= $this->get_post_content();
        }

        $result_prompt = $prompt . "\n\n" . $objective . "\n\n" . $notice . "\n\nQuestion:" . $question . "\n\nAnswer:\n\n";

        $complete = json_decode( $open_ai->completion( [
            'model' => esc_attr( $options['open_ai_model'] ),
            'prompt' => $result_prompt,
            'temperature' => (int)esc_attr( $options['open_ai_temperature'] ),
            'max_tokens' => (int)esc_attr( $options['open_ai_max_tokens'] ),
            'top_p' => 1,
            'frequency_penalty' => 0,
            'presence_penalty' => 0,
            'stop' => [
                "\nNote:",
                "\nQuestion:"
            ]
        ] ) );

        if ( isset( $complete->choices[0]->text ) ) {
            $text = str_replace( "\\n", "\n", $complete->choices[0]->text );
        } elseif ( isset( $complete->error->message ) ) {
            $text = current_user_can( 'editor' ) || current_user_can( 'administrator' ) ?
                $complete->error->message :
                esc_html__( 'Something went wrong. Try again later.', 'helper' );
        } else {
            $text = esc_html__( "Sorry, but I don't know how to answer that.", 'helper' );
        }


        return $text;
    }


    /**
     * Main Caster Instance.
     * Insures that only one instance of OpenAiBot exists in memory at any one time.
     *
     * @static
     * @since 1.0.0
     * @access public
     *
     * @return OpenAiBot
     **/
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {

            self::$instance = new self;

        }

        return self::$instance;

    }
}